// Copyright (c) 2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#ifndef XMLPARSER_XML_HANDLER_BASE_HPP_
#define XMLPARSER_XML_HANDLER_BASE_HPP_

/**
	@file xml-handler-base.hpp
	Base XML document class
*/

#include "xml-attr.hpp"

#include <vector>


namespace simplexml
{

/**
	@addtogroup xmlgroup
	@{
 */

class XMLValidator;

/**
	XML handler interface.
 */
class XMLHandlerBase
{

public:
	/**
		Start of XML Element
		@param element - [in] tag name
		@param level - [in] tag level
		@param attrs - [in] tag attributes
		@param line - [in] line
		@param column - [in] position
		@return 0 - ok, -1 - error
	 */
	virtual INT_32 StartElement(const std::string &element, UINT_32 level,
	                            const std::vector<simplexml::XMLAttr> &attrs,
	                            UINT_32 line, UINT_32 column) = 0;

	/**
		End of XML Element
		@param element - [in] tag name
		@param level - [in] tag level
		@param line - [in] line
		@param column - [in] position
		@return 0 - ok, -1 - error
	 */
	virtual INT_32 EndElement(const std::string &element, UINT_32 level,
	                          UINT_32 line, UINT_32 column) = 0;

	/**
		Characters data
		@param data - [in] character data
		@param level - [in] tag level
		@param line - [in] line
		@param column - [in] position
		@return 0 - ok, -1 - error
	 */
	virtual INT_32 Characters(const std::string &data, UINT_32 level,
	                          UINT_32 line, UINT_32 column) = 0;

	virtual ~XMLHandlerBase() throw() { ;; }

	/**
		Validate xml tree
		@param validator - validator
		@throws simplexml::XMLException
	 */
	virtual void Validate(const XMLValidator &validator) const = 0;

}; // class XMLHandlerBase

/** @} */

} // namespace simplexml

#endif // XMLPARSER_XML_HANDLER_BASE_HPP_
