// Copyright (c) 2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#ifndef XMLPARSER_XML_VALIDATOR_HPP_
#define XMLPARSER_XML_VALIDATOR_HPP_

/**
	@file xml-validator.hpp
	Generic XML validator class used by flint
*/

#include "xml-validator-base.hpp"

namespace simplexml
{

/**
	@addtogroup xmlgroup
	@{
	@addtogroup xmlvalidgroup XML Validator
	@{
 */

class XMLElement;

/**
	XML validating.
 */
class XMLValidator : public simplexml::XMLValidatorBase
{
public:
	typedef std::vector<simplexml::XMLElement *> ChildVec;
	typedef ChildVec::const_iterator ChildCIter;

	virtual ~XMLValidator() throw() = 0;

	/**
		Validate xml tree
		@param root - root of xml tree
		@throws simplexml::XMLException
	 */
	virtual void Validate(XMLElement *root) const;

protected:
	virtual const XMLElement *SpecRoot() const throw() { return NULL; }

private:
	/**
		Checks if tags differs.
	 */
	void DiffTag(const simplexml::XMLElement *spec,
	             simplexml::XMLElement *root,
	             const std::string &missingPath = "") const;

	/**
		Checks if attributes differs.
	 */
	void DiffAttr(const simplexml::XMLElement *spec,
	              simplexml::XMLElement *root,
	              const std::string &missingPath = "") const;

	/**
		Throws XMLValidatorException exception on unknown option.
	 */
	bool FindUnknownOption(const simplexml::XMLElement *spec,
	                       const std::string &option) const;


	/**
		Throws XMLValidatorException exception on unknown tag.
	 */
	void AssertNoUnknownTags(const simplexml::XMLElement *spec,
	                         simplexml::XMLElement *root) const;

	/**
		Throws XMLValidatorException exception on unknown attribute.
	 */
	void AssertNoUnknownAttrs(const simplexml::XMLElement *spec,
	                          simplexml::XMLElement *root) const;

}; // class XMLValidator

/** @} @} */

} // namespace simplexml

#endif // XMLPARSER_XML_VALIDATOR_HPP_

